@@ -276,7 +276,7 @@ impl App {
             .unwrap_or_else(env::temp_dir)
             .join("xplr")
             .join("session")
-            .join(&pid.to_string())
+            .join(pid.to_string())
             .to_string_lossy()
             .to_string();
 
@@ -284,7 +284,7 @@ impl App {
             session_path = env::temp_dir()
                 .join("xplr")
                 .join("session")
-                .join(&pid.to_string())
+                .join(pid.to_string())
                 .to_string_lossy()
                 .to_string();
             fs::create_dir_all(&session_path)?;
@@ -1105,14 +1105,11 @@ impl App {
             .and_then(App::reset_input_buffer)
     }
 
-    fn switch_mode_keeping_input_buffer(mut self, mode: &str) -> Result<Self> {
-        if let Some(mode) = self.config.modes.get(mode).cloned() {
-            self = self.push_mode();
-            self.mode = mode.sanitized(
-                self.config.general.read_only,
-                self.config.general.global_key_bindings.to_owned(),
-            );
-            Ok(self)
+    fn switch_mode_keeping_input_buffer(self, mode: &str) -> Result<Self> {
+        if self.config.modes.builtin.contains_key(mode) {
+            self.switch_mode_builtin_keeping_input_buffer(mode)
+        } else if self.config.modes.custom.contains_key(mode) {
+            self.switch_mode_custom_keeping_input_buffer(mode)
         } else {
             self.log_error(format!("Mode not found: {}", mode))
         }
@@ -1130,6 +1127,13 @@ impl App {
                 self.config.general.read_only,
                 self.config.general.global_key_bindings.to_owned(),
             );
+
+            // Hooks
+            if !self.hooks.on_mode_switch.is_empty() {
+                let msgs = self.hooks.on_mode_switch.clone();
+                self = self.handle_batch_external_msgs(msgs)?
+            }
+
             Ok(self)
         } else {
             self.log_error(format!("Builtin mode not found: {}", mode))
@@ -1148,16 +1152,24 @@ impl App {
                 self.config.general.read_only,
                 self.config.general.global_key_bindings.to_owned(),
             );
+
+            // Hooks
+            if !self.hooks.on_mode_switch.is_empty() {
+                let msgs = self.hooks.on_mode_switch.clone();
+                self = self.handle_batch_external_msgs(msgs)?
+            }
+
             Ok(self)
         } else {
             self.log_error(format!("Custom mode not found: {}", mode))
         }
     }
 
-    fn switch_layout(mut self, layout: &str) -> Result<Self> {
-        if let Some(l) = self.config.layouts.get(layout) {
-            self.layout = l.to_owned();
-            Ok(self)
+    fn switch_layout(self, layout: &str) -> Result<Self> {
+        if self.config.layouts.builtin.contains_key(layout) {
+            self.switch_layout_builtin(layout)
+        } else if self.config.layouts.custom.contains_key(layout) {
+            self.switch_layout_custom(layout)
         } else {
             self.log_error(format!("Layout not found: {}", layout))
         }
@@ -1166,6 +1178,13 @@ impl App {
     fn switch_layout_builtin(mut self, layout: &str) -> Result<Self> {
         if let Some(l) = self.config.layouts.builtin.get(layout) {
             self.layout = l.to_owned();
+
+            // Hooks
+            if !self.hooks.on_layout_switch.is_empty() {
+                let msgs = self.hooks.on_layout_switch.clone();
+                self = self.handle_batch_external_msgs(msgs)?
+            }
+
             Ok(self)
         } else {
             self.log_error(format!("Builtin layout not found: {}", layout))
@@ -1175,6 +1194,13 @@ impl App {
     fn switch_layout_custom(mut self, layout: &str) -> Result<Self> {
         if let Some(l) = self.config.layouts.get_custom(layout) {
             self.layout = l.to_owned();
+
+            // Hooks
+            if !self.hooks.on_layout_switch.is_empty() {
+                let msgs = self.hooks.on_layout_switch.clone();
+                self = self.handle_batch_external_msgs(msgs)?
+            }
+
             Ok(self)
         } else {
             self.log_error(format!("Custom layout not found: {}", layout))
