@@ -2,18 +2,20 @@ use crate::app::{
     DirectoryBuffer, ExplorerConfig, ExternalMsg, InternalMsg, MsgIn, Node, Task,
 };
 use anyhow::{Error, Result};
+use fuzzy_matcher::skim::SkimMatcherV2;
+use fuzzy_matcher::FuzzyMatcher;
+use lazy_static::lazy_static;
 use std::fs;
 use std::path::PathBuf;
 use std::sync::mpsc::Sender;
 use std::thread;
 
-pub(crate) fn explore_sync(
-    config: ExplorerConfig,
-    parent: PathBuf,
-    focused_path: Option<PathBuf>,
-    fallback_focus: usize,
-) -> Result<DirectoryBuffer> {
-    let dirs = fs::read_dir(&parent)?;
+lazy_static! {
+    static ref FUZZY_MATCHER: SkimMatcherV2 = SkimMatcherV2::default();
+}
+
+pub fn explore(parent: &PathBuf, config: &ExplorerConfig) -> Result<Vec<Node>> {
+    let dirs = fs::read_dir(parent)?;
     let mut nodes = dirs
         .filter_map(|d| {
             d.ok().map(|e| {
@@ -27,9 +29,36 @@ pub(crate) fn explore_sync(
         .filter(|n| config.filter(n))
         .collect::<Vec<Node>>();
 
-    nodes.sort_by(|a, b| config.sort(a, b));
+    nodes = if let Some(pattern) = config.searcher.as_ref().map(|s| &s.pattern) {
+        let mut nodes = nodes
+            .into_iter()
+            .filter_map(|n| {
+                FUZZY_MATCHER
+                    .fuzzy_match(&n.relative_path, pattern)
+                    .map(|score| (n, score))
+            })
+            .collect::<Vec<(_, _)>>();
 
-    let focus_index = if let Some(focus) = focused_path {
+        nodes.sort_by(|(_, s1), (_, s2)| s2.cmp(s1));
+        nodes.into_iter().map(|(n, _)| n).collect::<Vec<_>>()
+    } else {
+        nodes.sort_by(|a, b| config.sort(a, b));
+        nodes
+    };
+
+    Ok(nodes)
+}
+
+pub(crate) fn explore_sync(
+    config: ExplorerConfig,
+    parent: PathBuf,
+    focused_path: Option<PathBuf>,
+    fallback_focus: usize,
+) -> Result<DirectoryBuffer> {
+    let nodes = explore(&parent, &config)?;
+    let focus_index = if config.searcher.is_some() {
+        0
+    } else if let Some(focus) = focused_path {
         let focus_str = focus.to_string_lossy().to_string();
         nodes
             .iter()
