@@ -174,9 +174,6 @@ pub enum ExternalMsg {
     /// - YAML: `FocusByIndexFromInput`
     FocusByIndexFromInput,
 
-    ///
-    /// **YAML:** `FocusByFileName: string`
-    ///
     /// Focus on the file by name from the present working directory.
     ///
     /// Type: { FocusByFileName = "string" }
@@ -187,6 +184,38 @@ pub enum ExternalMsg {
     /// - YAML: `FocusByFileName: filename.ext`
     FocusByFileName(String),
 
+    /// Scroll up by terminal height.
+    ///
+    /// Example:
+    ///
+    /// - Lua: `"ScrollUp"`
+    /// - YAML: `ScrollUp`
+    ScrollUp,
+
+    /// Scroll down by terminal height.
+    ///
+    /// Example:
+    ///
+    /// - Lua: `"ScrollDown"`
+    /// - YAML: `ScrollDown`
+    ScrollDown,
+
+    /// Scroll up by half of terminal height.
+    ///
+    /// Example:
+    ///
+    /// - Lua: `"ScrollUpHalf"`
+    /// - YAML: `ScrollUpHalf`
+    ScrollUpHalf,
+
+    /// Scroll down by half of terminal height.
+    ///
+    /// Example:
+    ///
+    /// - Lua: `"ScrollDownHalf"`
+    /// - YAML: `ScrollDownHalf`
+    ScrollDownHalf,
+
     /// Change the present working directory ($PWD)
     ///
     /// Type: { ChangeDirectory = "string" }
@@ -229,15 +258,53 @@ pub enum ExternalMsg {
     /// - YAML: `NextVisitedPath`
     NextVisitedPath,
 
-    ///
     /// Follow the symlink under focus to its actual location.
     ///
     /// Example:
     ///
-    /// Lua: `"FollowSymlink"`
-    /// YAML: `FollowSymlink`
+    /// - Lua: `"FollowSymlink"`
+    /// - YAML: `FollowSymlink`
     FollowSymlink,
 
+    /// ### Virtual Root ------------------------------------------------------
+
+    /// Sets the virtual root for isolating xplr navigation, similar to
+    /// `--vroot`, but temporary (can be reset back to initial value).
+    /// If the $PWD is outside the vroot, xplr will automatically enter vroot.
+    ///
+    /// Type: { SetVroot = "string" }
+    ///
+    /// Example:
+    ///
+    /// - Lua: `{ SetVroot = "/tmp" }`
+    /// - YAML: `SetVroot: /tmp`
+    SetVroot(String),
+
+    /// Unset the virtual root temporarily (can be reset back to the initial
+    /// value).
+    ///
+    /// Example:
+    ///
+    /// - Lua: `"UnsetVroot"`
+    /// - YAML: `UnsetVroot`
+    UnsetVroot,
+
+    /// Toggle virtual root between unset, initial value and $PWD.
+    ///
+    /// Example:
+    ///
+    /// - Lua: `"ToggleVroot"`
+    /// - YAML: `ToggleVroot`
+    ToggleVroot,
+
+    /// Reset the virtual root back to the initial value.
+    ///
+    /// Example:
+    ///
+    /// - Lua: `"ResetVroot"`
+    /// - YAML: `ResetVroot`
+    ResetVroot,
+
     /// ### Reading Input -----------------------------------------------------
 
     /// Set the input prompt temporarily, until the input buffer is reset.
@@ -335,8 +402,7 @@ pub enum ExternalMsg {
     /// - YAML: SwitchMode: default
     ///
     /// > **NOTE:** To be specific about which mode to switch to, use
-    /// > `SwitchModeBuiltinKeepingInputBuffer` or
-    /// > `SwitchModeCustomKeepingInputBuffer` instead.
+    /// > `SwitchModeBuiltin` or `SwitchModeCustom` instead.
     SwitchMode(String),
 
     /// Switch input [mode](https://xplr.dev/en/modes).
@@ -453,52 +519,72 @@ pub enum ExternalMsg {
 
     /// ### Executing Commands ------------------------------------------------
 
+    /// Like `Call0`, but it uses `\n` as the delimiter in input/output pipes,
+    /// hence it cannot handle files with `\n` in the name.
+    /// You may want to use `Call0` instead.
+    Call(Command),
+
     /// Call a shell command with the given arguments.
     /// Note that the arguments will be shell-escaped.
     /// So to read the variables, the `-c` option of the shell
     /// can be used.
-    /// You may need to pass `ExplorePwd` depening on the expectation.
+    /// You may need to pass `ExplorePwd` depending on the expectation.
     ///
-    /// Type: { Call = { command = string, args = { "list", "of", "string" } }
+    /// Type: { Call0 = { command = "string", args = { "list", "of", "string" } }
     ///
     /// Example:
     ///
-    /// - Lua: `{ Call = { command = "bash", args = { "-c", "read -p test" } } }`
-    /// - YAML: `Call: { command: bash, args: ["-c", "read -p test"] }`
-    Call(Command),
+    /// - Lua: `{ Call0 = { command = "bash", args = { "-c", "read -p test" } } }`
+    /// - YAML: `Call0: { command: bash, args: ["-c", "read -p test"] }`
+    Call0(Command),
 
-    /// Like `Call` but without the flicker. The stdin, stdout
+    /// Like `CallSilently0`, but it uses `\n` as the delimiter in input/output
+    /// pipes, hence it cannot handle files with `\n` in the name.
+    /// You may want to use `CallSilently0` instead.
+    CallSilently(Command),
+
+    /// Like `Call0` but without the flicker. The stdin, stdout
     /// stderr will be piped to null. So it's non-interactive.
     ///
-    /// Type: { CallSilently = "string" }
+    /// Type: { CallSilently0 = { command = "string", args = {"list", "of", "string"} } }
     ///
     /// Example:
     ///
-    /// - Lua: `{ CallSilently = { command = "tput", args = { "bell" } } }`
-    /// - YAML: `CallSilently: { command: tput, args: ["bell"] }`
-    CallSilently(Command),
+    /// - Lua: `{ CallSilently0 = { command = "tput", args = { "bell" } } }`
+    /// - YAML: `CallSilently0: { command: tput, args: ["bell"] }`
+    CallSilently0(Command),
+
+    /// Like `BashExec0`, but it uses `\n` as the delimiter in input/output
+    /// pipes, hence it cannot handle files with `\n` in the name.
+    /// You may want to use `BashExec0` instead.
+    BashExec(String),
 
     /// An alias to `Call: {command: bash, args: ["-c", "{string}"], silent: false}`
     /// where `{string}` is the given value.
     ///
-    /// Type: { BashExec = "string" }
+    /// Type: { BashExec0 = "string" }
     ///
     /// Example:
     ///
-    /// - Lua: `{ BashExec = "read -p test" }`
-    /// - YAML: `BashExec: "read -p test"`
-    BashExec(String),
+    /// - Lua: `{ BashExec0 = "read -p test" }`
+    /// - YAML: `BashExec0: "read -p test"`
+    BashExec0(String),
 
-    /// Like `BashExec` but without the flicker. The stdin, stdout
+    /// Like `BashExecSilently0`, but it uses `\n` as the delimiter in
+    /// input/output pipes, hence it cannot handle files with `\n` in the name.
+    /// You may want to use `BashExecSilently0` instead.
+    BashExecSilently(String),
+
+    /// Like `BashExec0` but without the flicker. The stdin, stdout
     /// stderr will be piped to null. So it's non-interactive.
     ///
-    /// Type: { BashExecSilently = "string" }
+    /// Type: { BashExecSilently0 = "string" }
     ///
     /// Example:
     ///
-    /// - Lua: `{ BashExecSilently = "tput bell" }`
-    /// - YAML: `BashExecSilently: "tput bell"`
-    BashExecSilently(String),
+    /// - Lua: `{ BashExecSilently0 = "tput bell" }`
+    /// - YAML: `BashExecSilently0: "tput bell"`
+    BashExecSilently0(String),
 
     /// ### Calling Lua Functions ----------------------------------------------
 
@@ -646,6 +732,8 @@ pub enum ExternalMsg {
 
     /// Add a [filter](https://xplr.dev/en/filtering#filter) to exclude nodes
     /// while exploring directories.
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
+    /// Filters get automatically cleared when changing directories.
     ///
     /// Type: { AddNodeFilter = { filter = [Filter](https://xplr.dev/en/filtering#filter), input = "string" }
     ///
@@ -656,6 +744,7 @@ pub enum ExternalMsg {
     AddNodeFilter(NodeFilterApplicable),
 
     /// Remove an existing [filter](https://xplr.dev/en/filtering#filter).
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Type: { RemoveNodeFilter = { filter = [Filter](https://xplr.dev/en/filtering), input = "string" }
     ///
@@ -667,6 +756,7 @@ pub enum ExternalMsg {
 
     /// Remove a [filter](https://xplr.dev/en/filtering#filter) if it exists,
     /// else, add a it.
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Type: { ToggleNodeFilter = { filter = [Filter](https://xplr.dev/en/filtering), input = "string" }
     ///
@@ -678,6 +768,7 @@ pub enum ExternalMsg {
 
     /// Add a node [filter](https://xplr.dev/en/filtering#filter) reading the
     /// input from the buffer.
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Type: { AddNodeFilterFromInput = [Filter](https://xplr.dev/en/filtering) }
     ///
@@ -689,6 +780,7 @@ pub enum ExternalMsg {
 
     /// Remove a node [filter](https://xplr.dev/en/filtering#filter) reading
     /// the input from the buffer.
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Type: { RemoveNodeFilterFromInput = [Filter](https://xplr.dev/en/filtering) }
     ///
@@ -699,6 +791,7 @@ pub enum ExternalMsg {
     RemoveNodeFilterFromInput(NodeFilter),
 
     /// Remove the last node [filter](https://xplr.dev/en/filtering).
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Example:
     ///
@@ -708,6 +801,7 @@ pub enum ExternalMsg {
 
     /// Reset the node [filters](https://xplr.dev/en/filtering) back to the
     /// default configuration.
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Example:
     ///
@@ -716,6 +810,7 @@ pub enum ExternalMsg {
     ResetNodeFilters,
 
     /// Clear all the node [filters](https://xplr.dev/en/filtering).
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Example:
     ///
@@ -727,6 +822,7 @@ pub enum ExternalMsg {
 
     /// Add a [sorter](https://xplr.dev/en/sorting#sorter) to sort nodes while
     /// exploring directories.
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Type: { AddNodeSorter = { sorter = [Sorter](https://xplr.dev/en/sorting#sorter), reverse = bool } }
     ///
@@ -737,6 +833,7 @@ pub enum ExternalMsg {
     AddNodeSorter(NodeSorterApplicable),
 
     /// Remove an existing [sorter](https://xplr.dev/en/sorting#sorter).
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Type: { RemoveNodeSorter = [Sorter](https://xplr.dev/en/sorting#sorter) }
     ///
@@ -747,6 +844,7 @@ pub enum ExternalMsg {
     RemoveNodeSorter(NodeSorter),
 
     /// Reverse a node [sorter](https://xplr.dev/en/sorting#sorter).
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Type: { ReverseNodeSorter = [Sorter](https://xplr.dev/en/sorting#sorter) }
     ///
@@ -758,6 +856,7 @@ pub enum ExternalMsg {
 
     /// Remove a [sorter](https://xplr.dev/en/sorting#sorter) if it exists,
     /// else, add a it.
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Type: { ToggleNodeSorter = { sorter = [Sorter](https://xplr.dev/en/sorting#sorter), reverse = bool } }
     ///
@@ -768,6 +867,7 @@ pub enum ExternalMsg {
     ToggleNodeSorter(NodeSorterApplicable),
 
     /// Reverse the node [sorters](https://xplr.dev/en/sorting#sorter).
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Example:
     ///
@@ -776,6 +876,7 @@ pub enum ExternalMsg {
     ReverseNodeSorters,
 
     /// Remove the last node [sorter](https://xplr.dev/en/sorting#sorter).
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Example:
     ///
@@ -785,6 +886,7 @@ pub enum ExternalMsg {
 
     /// Reset the node [sorters](https://xplr.dev/en/sorting#sorter) back to
     /// the default configuration.
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Example:
     ///
@@ -793,6 +895,7 @@ pub enum ExternalMsg {
     ResetNodeSorters,
 
     /// Clear all the node [sorters](https://xplr.dev/en/sorting#sorter).
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
     ///
     /// Example:
     ///
@@ -800,6 +903,49 @@ pub enum ExternalMsg {
     /// - YAML: `ClearNodeSorters`
     ClearNodeSorters,
 
+    /// ### Search Operations --------------------------------------------------
+
+    /// Search files using fuzzy match algorithm.
+    /// It keeps the filters, but overrides the sorters.
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
+    /// It gets reset automatically when changing directory.
+    ///
+    /// Type: { SearchFuzzy = "string" }
+    ///
+    /// Example:
+    ///
+    /// - Lua: `{ SearchFuzzy = "pattern" }`
+    /// - YAML: `SearchFuzzy: pattern`
+    SearchFuzzy(String),
+
+    /// Calls `SearchFuzzy` with the input taken from the input buffer.
+    /// You need to call `ExplorePwd` or `ExplorePwdAsync` explicitely.
+    ///
+    /// Example:
+    ///
+    /// - Lua: `"SearchFuzzyFromInput"`
+    /// - YAML: `SearchFuzzyFromInput`
+    SearchFuzzyFromInput,
+
+    /// Accepts the search by keeping the latest focus while in search mode.
+    /// Automatically calls `ExplorePwd`.
+    ///
+    /// Example:
+    ///
+    /// - Lua: `"AcceptSearch"`
+    /// - YAML: `AcceptSearch`
+    AcceptSearch,
+
+    /// Cancels the search by discarding the latest focus and recovering
+    /// the focus before search.
+    /// Automatically calls `ExplorePwd`.
+    ///
+    /// Example:
+    ///
+    /// - Lua: `"CancelSearch"`
+    /// - YAML: `CancelSearch`
+    CancelSearch,
+
     /// ### Mouse Operations ---------------------------------------------------
 
     /// Enable mouse
@@ -979,9 +1125,13 @@ impl ExternalMsg {
         !matches!(
             self,
             Self::Call(_)
+                | Self::Call0(_)
                 | Self::CallSilently(_)
+                | Self::CallSilently0(_)
                 | Self::BashExec(_)
+                | Self::BashExec0(_)
                 | Self::BashExecSilently(_)
+                | Self::BashExecSilently0(_)
                 | Self::CallLua(_)
                 | Self::CallLuaSilently(_)
                 | Self::LuaEval(_)
@@ -1484,10 +1634,33 @@ impl NodeFilterApplicable {
     }
 }
 
+#[derive(Debug, Clone, Eq, Hash, PartialEq, Serialize, Deserialize)]
+pub struct NodeSearcher {
+    pub pattern: String,
+
+    #[serde(default)]
+    pub recoverable_focus: Option<String>,
+}
+
+impl NodeSearcher {
+    pub fn new(pattern: String, recoverable_focus: Option<String>) -> Self {
+        Self {
+            pattern,
+            recoverable_focus,
+        }
+    }
+}
+
 #[derive(Debug, Default, Clone, Eq, PartialEq, Serialize, Deserialize)]
 pub struct ExplorerConfig {
+    #[serde(default)]
     pub filters: IndexSet<NodeFilterApplicable>,
+
+    #[serde(default)]
     pub sorters: IndexSet<NodeSorterApplicable>,
+
+    #[serde(default)]
+    pub searcher: Option<NodeSearcher>,
 }
 
 impl ExplorerConfig {
